
%***********************************The MoDem****************************

function [number_of_errors,bit_error_rate]=Simulasjon_hele(snr,Phase)
%%----------------------------------Set parameters-----------------------
M = 4; % Size of signal constellation
k = log2(M); % Number of bits per channel symbol
n = 1016; % Number of bits to process (size of the maximum packet)

%%----------------------------------Signal Source------------------------
% Random number generator
x = randint(n,1); 

%%----------------------------------Modulator----------------------------
%%Bit-to-symbol mapping
xsmb=reshape(x,4,length(x)/4).';

%%symbol-to-chip mapping in bits
[l1,l2]=size(xsmb);
for l=1:l1
    [yI(l,:),yQ(l,:)]=mapping(xsmb(l,:));
end
yI=yI';
yI=yI(:);
yQ=yQ';
yQ=yQ(:);

%QPSK modulation
yI=yI*2 - 1; % I branch mapped from 0/1 to -1/1
yQ=yQ*2 - 1; % Q branch mapped from 0/1 to -1/1
y=yI+j*yQ;

%Gard interval 
y=[0 0 y' 0 0]';

% Upsample and apply the 8 tap half-sine pulse-shaping filter.
y=upsample(y,8);
ytx=shape2(y);

%%---------------------------------Channel-------------------------------
% Send signal over an AWGN channel.
ynoisy = awgn(ytx,snr,'measured');

%%---------------------------------Demodulator---------------------------
%quantize the input of the demodulator down to 4 bits before the channel   
%filter for case 3. Equivalent to using 4 bits ADC. 
ynoisy=kvantiser4(real(ynoisy))+j*kvantiser4(imag(ynoisy));

%Channel filter used in case 1 and 2
% yrx = kanalfilter(ynoisy);

%Channel filter is used in case 3. It has approximately twice times as 
%many taps as the filter used in case 1 and 2.
yrx = kanalfilter2(ynoisy);

% Downsample the output of the channel filter
yrx=downsample(yrx,2,1);
k=length(yrx);

offset=freqoffset(snr); %Get the frequency offset for the given SNR
for r=1:k
    if r<k/5
        n=(r *offset)+Phase; %Phase offset at time instance r
        yrx(r)=yrx(r)*exp(j*n);
    end
    if r>=k/5 & r<=2*k/5
        n=((r-k/5) *offset)+Phase; %Phase offset at time instance r,                      
                     %with compensated phase at 1/5 the packet length  
        yrx(r)=yrx(r)*exp(j*n);
    end
    if r>=2*k/5 & r<=3*k/5
        n=((r-2*k/5) *offset)+Phase; %Phase offset at time instance r,                      
                       %with compensated phase at 2/5 the packet length  
        yrx(r)=yrx(r)*exp(j*n);
    end
    if r>=3*k/5 & r<=4*k/5
        n=((r-3*k/5) *offset)+Phase; %Phase offset at time instance r,                      
                       %with compensated phase at 3/5 the packet length  
        yrx(r)=yrx(r)*exp(j*n);
    end
    if r>=4*k/5
        n=((r-4*k/5) *offset)+Phase; %Phase offset at time instance r,                      
                      % with compensated phase at 4/5 the packet length  
        yrx(r)=yrx(r)*exp(j*n);
    end
end


% Matched filter. Apply the 4 tap half-sine pulse-shaping filter.
yrx = shape(yrx);

%Frame synchronization 
%For Case 3
yrx = yrx(17:end);
%For Case 1 and 2
% yrx = yrx(14:end-3);

% Divide into two paths
yrxI= real(yrx);
yrxQ =imag(yrx); 

%Quantize to 2 bits for case 2 and 3
yrxI=kvantiser(yrxI);
yrxQ=kvantiser(yrxQ);

%Group the sequence so the correlator can take in one and one symbol
l=length(yrx)/64;
yrxI=reshape(yrxI,64,l).';
yrxQ=reshape(yrxQ,64,l).';

%Demodulate the signal in the correlator
for r=1:l
z(r,:)=korrelator2(yrxI(r,:),yrxQ(r,:));
end

%reshape the matrix to a bitstream.
z;
z=z';
z=z(:);

%%---------------------------------BER Computation-----------------------
% Compare x and z to obtain the number of errors and the bit error rate.
[number_of_errors,bit_error_rate] = biterr(x,z);


%*************************The Symbol-to-Chip Mapping*********************
function [yI,yQ]=mapping(x)

%The 0-symbol. All other symbols are given by shifts or inversions 
seq=[1 1 0 1 1 0 0 1 1 1 0 0 0 0 1 1 0 1 0 1 0 0 1 0 0 0 1 0 1 1 1 0];

%Divide into the I and Q information
seqI=seq(1:2:end);
seqQ=seq(2:2:end);

if x==[0 0 0 0]  %0-symbol
    yI=seqI;
    yQ=seqQ;
elseif x==[1 0 0 0] %1-symbol
        yI=circshift(seqI,[0 2]);
        yQ=circshift(seqQ,[0 2]);
elseif x==[0 1 0 0] %2-symbol
        yI=circshift(seqI,[0 4]);
        yQ=circshift(seqQ,[0 4]);
elseif x==[1 1 0 0] %3-symbol
        yI=circshift(seqI,[0 6]);
        yQ=circshift(seqQ,[0 6]);
elseif x==[0 0 1 0] %4-symbol
        yI=circshift(seqI,[0 8]);
        yQ=circshift(seqQ,[0 8]);
elseif x==[1 0 1 0] %5-symbol
        yI=circshift(seqI,[0 10]);
        yQ=circshift(seqQ,[0 10]);
elseif x==[0 1 1 0] %6-symbol
        yI=circshift(seqI,[0 12]);
        yQ=circshift(seqQ,[0 12]);
elseif x==[1 1 1 0] %7-symbol
        yI=circshift(seqI,[0 14]);
        yQ=circshift(seqQ,[0 14]);
elseif x==[0 0 0 1] %8-symbol
    yI=seqI;
    yQ=~seqQ;
elseif  x==[1 0 0 1] %9-symbol
        yI=circshift(seqI,[0 2]);
        yQ=~circshift(seqQ,[0 2]);
elseif  x==[0 1 0 1] %10-symbol
        yI=circshift(seqI,[0 4]);
        yQ=~circshift(seqQ,[0 4]);
elseif  x==[1 1 0 1] %11-symbol
        yI=circshift(seqI,[0 6]);
        yQ=~circshift(seqQ,[0 6]);
elseif  x==[0 0 1 1] %12-symbol
        yI=circshift(seqI,[0 8]);
        yQ=~circshift(seqQ,[0 8]);
elseif  x==[1 0 1 1] %13-symbol
        yI=circshift(seqI,[0 10]);
        yQ=~circshift(seqQ,[0 10]);
elseif  x==[0 1 1 1] %14-symbol
        yI=circshift(seqI,[0 12]);
        yQ=~circshift(seqQ,[0 12]);
elseif  x==[1 1 1 1] %15-symbol
        yI=circshift(seqI,[0 14]);
        yQ=~circshift(seqQ,[0 14]);
end

%************************%8 tap half-sine pulse-shaping filter***********
function y = shape2(x);
%Filter coefficients from a sampled half-sine
filt = [0 0.3827  .7071  0.9239 1  0.9239 .7071 0.3827];
y = filter(filt,1,x) ;

%************************%4 bits Quantizer*******************************
function [w] = kvantiser4(y)
%Returns the 4 bits quantized value

%partition
q = [-0.9334 -0.8000 -0.6667 -0.5333 -0.4000 -0.2667 -0.1334 0 ...
    0.1334 0.2667 0.4000 0.5333 0.6667 0.8000 0.9334];

%codebook
c = [-1.0000 -0.8667 -0.7333 -0.6000 -0.4667 -0.3333 -0.2000 -0.0667 ...
    0.0667 0.2000 0.3333 0.4667 0.6000 0.7333 0.8667 1.0000];

[index,w] = quantiz(y,q,c);

%************************%4 bits Channel Filter for Case 3***************
function y = kanalfilter2(x)
%Returns the low-pass filtered signal.

% All frequency values are in KHz.
Fs = 8000;  % Sampling Frequency

N     = 23;    % Order
Fpass = 820;   % Passband Frequency
Fstop = 1520;  % Stopband Frequency
Wpass = 1;     % Passband Weight
Wstop = 1;     % Stopband Weight
dens  = 20;    % Density Factor

% Calculate the coefficients using the FIRPM function.
bb  = firpm(N, [0 Fpass Fstop Fs/2]/(Fs/2), [1 1 0 0], [Wpass Wstop], ...
           {dens});

b=kvantiser4(bb); %4 bit resolution on the coefficients
y=filter(b,1,x);

%************************%Unquantized Channel Filter for Case 1 and 2****
function y= kanalfilter(x);
%Returns the low-pass filtered signal.

% All frequency values are in kHz.
Fs = 8000;  % Sampling Frequency

N     = 10;    % Order
Fpass = 820;   % Passband Frequency
Fstop = 2500;  % Stopband Frequency
Wpass = 1;     % Passband Weight
Wstop = 1;     % Stopband Weight
dens  = 20;    % Density Factor

% Calculate the coefficients using the FIRPM function.
b  = firpm(N, [0 Fpass Fstop Fs/2]/(Fs/2), [1 1 0 0], [Wpass Wstop], ...
           {dens});

y=filter(b,1,x);

%************************%Frequency Offset****************************
function offset=freqoffset(snr)
%Returns a frequency offset as a function of SNR
offset=sqrt(4*10^-8 -(1.98e-009)*(snr+9.8));

%************************%4 tapped Matched Filter ***********************
function y = shape(x);
%Create the 4 tap half-sine pulse-shaping filter
filt = [0 .7071 1 .7071 ];
y = filter(filt,1,x) ;


%************************%2 bits quantizer *************************

function [w] = kvantiser(y)
%Returns the 2 bits quantized value

%partition
q = [-0.5 0.5];

%codebook
c = [-1 0 1];
[index,w] = quantiz(y,q,c);

%************************%Correlator *************************
function det= korrelator2(xI,xQ)

%The 0-symbol
R = [1 1 0 1 1 0 0 1 1 1 0 0 0 0 1 1 0 1 0 1 0 0 1 0 0 0 1 0 1 1 1 0];
R = 2*R - 1;
%-----------------The I Correlator Information---------------------------
rI = R(1:2:end);

%The I branch information of the 
rI1 = rI; % 0/8-symbol
rI2 = circshift(rI,[0 2]); % 1/9-symbol
rI3 = circshift(rI,[0 4]); % 2/10-symbol
rI4 = circshift(rI,[0 6]); % 3/11-symbol
rI5 = circshift(rI,[0 8]); % 4/12-symbol
rI6 = circshift(rI,[0 10]);% 5/13-symbol
rI7 = circshift(rI,[0 12]);% 6/14-symbol
rI8 = circshift(rI,[0 14]);% 7/15-symbol


%-----------------The Q Correlator Information---------------------------
rQ=R(2:2:end);

% rQ=rQ*2 -1; %0/1 to -1/1 mapping
rQ1=rQ; %0-symbol
rQ2=circshift(rQ,[0 2]); %1-symbol
rQ3=circshift(rQ,[0 4]); %2-symbol
rQ4=circshift(rQ,[0 6]); %3-symbol
rQ5=circshift(rQ,[0 8]); %4-symbol
rQ6=circshift(rQ,[0 10]);%5-symbol
rQ7=circshift(rQ,[0 12]);%6-symbol
rQ8=circshift(rQ,[0 14]);%7-symbol
rQ9=-rQ1;                %8-symbol
rQ10=-rQ2;	             %9-symbol
rQ11=-rQ3;	             %10-symbol
rQ12=-rQ4;               %11-symbol
rQ13=-rQ5;               %12-symbol
rQ14=-rQ6;               %13-symbol
rQ15=-rQ7;               %14-symbol
rQ16=-rQ8;               %15-symbol

%-----------------The I Correlator --------------------------------------
%distribute the incoming sequence on the 4 branches for                      
%finding best timing
xI1=xI(1:4:end);
xI2=xI(2:4:end);
xI3=xI(3:4:end);
xI4=xI(4:4:end);

detI1(1)=sum (xI1 .* rI1);
detI1(2)=sum (xI2 .* rI1);
detI1(3)=sum (xI3 .* rI1);
detI1(4)=sum (xI4 .* rI1);
det(1)=max(detI1);            % 0/8-symbol Correlator value

detI2(1)=sum (xI1 .* rI2);
detI2(2)=sum (xI2 .* rI2);
detI2(3)=sum (xI3 .* rI2);
detI2(4)=sum (xI4 .* rI2);
det(2)=max(detI2);            % 1/9-symbol Correlator value  

detI3(1)=sum (xI1 .* rI3) ;
detI3(2)=sum (xI2 .* rI3) ;
detI3(3)=sum (xI3 .* rI3) ;
detI3(4)=sum (xI4 .* rI3) ;
det(3)=max(detI3);            % 2/10-symbol Correlator value

detI4(1)=sum (xI1 .* rI4) ;
detI4(2)=sum (xI2 .* rI4) ;
detI4(3)=sum (xI3 .* rI4) ;
detI4(4)=sum (xI4 .* rI4) ;
det(4)=max(detI4);            % 3/11-symbol Correlator value

detI5(1)=sum (xI1 .* rI5) ;
detI5(2)=sum (xI2 .* rI5) ;
detI5(3)=sum (xI3 .* rI5) ;
detI5(4)=sum (xI4 .* rI5) ;
det(5)=max(detI5);            % 4/12-symbol Correlator value

detI6(1)=sum (xI1 .* rI6) ;
detI6(2)=sum (xI2 .* rI6) ;
detI6(3)=sum (xI3 .* rI6) ;
detI6(4)=sum (xI4 .* rI6) ;
det(6)=max(detI6);            % 5/13-symbol Correlator value

detI7(1)=sum (xI1 .* rI7) ;
detI7(2)=sum (xI2 .* rI7) ;
detI7(3)=sum (xI3 .* rI7) ;
detI7(4)=sum (xI4 .* rI7) ;
det(7)=max(detI7);            % 6/14-symbol Correlator value

detI8(1)=sum (xI1 .* rI8) ;
detI8(2)=sum (xI2 .* rI8) ;
detI8(3)=sum (xI3 .* rI8) ;
detI8(4)=sum (xI4 .* rI8) ;
det(8)=max(detI8);            % 7/15-symbol Correlator value


%Find biggest correlation value
maks = find(det==max(det));
s=length(maks);

%If more than 1 value is the biggest, choose randomly.
if s>=2
    maks=maks(ceil(rand()*s));
end
 
%-----------------The Q Correlator --------------------------------------
% xQ=xQ*2 -1; %0/1 to -1/1 mapping

%distribute the incoming sequence on the 4 branches for finding best 
%timing
xQ1=xQ(1:4:end);
xQ2=xQ(2:4:end);
xQ3=xQ(3:4:end);
xQ4=xQ(4:4:end);

%Check the sign of correlation value of Q-symbol number (maks-1) 
if maks ==1
    detQ_sum(1)=sum(xQ1 .* rQ1);
    detQ_sum(2)=sum(xQ2 .* rQ1);
    detQ_sum(3)=sum(xQ3 .* rQ1);
    detQ_sum(4)=sum(xQ4 .* rQ1);
    detQ=sign(detQ_sum(find(max(abs(detQ_sum))))) ;
elseif maks ==2
    detQ_sum(1)=sum(xQ1 .* rQ2);
    detQ_sum(2)=sum(xQ2 .* rQ2);
    detQ_sum(3)=sum(xQ3 .* rQ2);
    detQ_sum(4)=sum(xQ4 .* rQ2);
    detQ=sign(detQ_sum(find(max(abs(detQ_sum)))));
elseif maks ==3
    detQ_sum(1)=sum(xQ1 .* rQ3);
    detQ_sum(2)=sum(xQ2 .* rQ3);
    detQ_sum(3)=sum(xQ3 .* rQ3);
    detQ_sum(4)=sum(xQ4 .* rQ3);
    detQ=sign(detQ_sum(find(max(abs(detQ_sum)))));
elseif maks ==4
    detQ_sum(1)=sum(xQ1 .* rQ4);
    detQ_sum(2)=sum(xQ2 .* rQ4);
    detQ_sum(3)=sum(xQ3 .* rQ4);
    detQ_sum(4)=sum(xQ4 .* rQ4);
    detQ=sign(detQ_sum(find(max(abs(detQ_sum)))));
elseif maks ==5
    detQ_sum(1)=sum(xQ1 .* rQ5);
    detQ_sum(2)=sum(xQ2 .* rQ5);
    detQ_sum(3)=sum(xQ3 .* rQ5);
    detQ_sum(4)=sum(xQ4 .* rQ5);
    detQ=sign(detQ_sum(find(max(abs(detQ_sum)))));
elseif maks ==6
    detQ_sum(1)=sum(xQ1 .* rQ6);
    detQ_sum(2)=sum(xQ2 .* rQ6);
    detQ_sum(3)=sum(xQ3 .* rQ6);
    detQ_sum(4)=sum(xQ4 .* rQ6);
    detQ=sign(detQ_sum(find(max(abs(detQ_sum)))));
elseif maks ==7
    detQ_sum(1)=sum(xQ1 .* rQ7);
    detQ_sum(2)=sum(xQ2 .* rQ7);
    detQ_sum(3)=sum(xQ3 .* rQ7);
    detQ_sum(4)=sum(xQ4 .* rQ7);
    detQ=sign(detQ_sum(find(max(abs(detQ_sum)))));
elseif maks ==8
    detQ_sum(1)=sum(xQ1 .* rQ8);
    detQ_sum(2)=sum(xQ2 .* rQ8);
    detQ_sum(3)=sum(xQ3 .* rQ8);
    detQ_sum(4)=sum(xQ4 .* rQ8);
    detQ=sign(detQ_sum(find(max(abs(detQ_sum)))));
end


if length(detQ)==0
    detQ=0 ;
end
 
%The signum function is not defined for 0
if detQ==0
    detQ=2*randint()-1 ; 
end

%Detect symbol
if detQ==1
    det1=maks-1 ;
elseif detQ==-1
    det1=maks+7 ;
end

%Mapp from symbol number to bit values and return the detected
if det1==0
    det=[0 0 0 0];
elseif det1==1
    det=[0 0 0 1];
elseif det1==2
    det=[0 0 1 0];
elseif det1==3
    det=[0 0 1 1];
elseif det1==4
    det=[0 1 0 0];
elseif det1==5
    det=[0 1 0 1];
elseif det1==6
    det=[0 1 1 0];
elseif det1==7
    det=[0 1 1 1];
elseif det1==8
    det=[1 0 0 0];
elseif det1==9
    det=[1 0 0 1];
elseif det1==10
    det=[1 0 1 0];
elseif det1==11
    det=[1 0 1 1];
elseif det1==12
    det=[1 1 0 0];
elseif det1==13
    det=[1 1 0 1];
elseif det1==14
    det=[1 1 1 0];
elseif det1==15
    det=[1 1 1 1];
end
  
det=fliplr(det);








